@@ -6,40 +6,35 @@
 */
 
 #include "kdirlistertest.h"
+
+#include "jobuidelegatefactory.h"
+#include "kiotesthelper.h"
+#include <KDirWatch>
+#include <kio/copyjob.h>
+#include <kio/deletejob.h>
+#include <kio/job.h>
+#include <kio/jobuidelegateextension.h>
+#include <kprotocolinfo.h>
+
+#include <QDebug>
 #include <QTemporaryFile>
 #include <QTest>
 
 QTEST_MAIN(KDirListerTest)
 
-#include "kiotesthelper.h"
-#include <KDirWatch>
-#include <QDebug>
-#include <kio/copyjob.h>
-#include <kio/deletejob.h>
-#include <kio/job.h>
-#include <kprotocolinfo.h>
+static constexpr bool s_workaroundBrokenInotify = false;
 
-#define WORKAROUND_BROKEN_INOTIFY 0
-
-void MyDirLister::setErrorExpected()
+GlobalInits::GlobalInits()
 {
-    m_errorExpected = true;
+    // Must be done before the QSignalSpys connect
+    qRegisterMetaType<KFileItem>();
+    qRegisterMetaType<KFileItemList>();
+    qRegisterMetaType<KIO::Job *>();
 }
 
-int MyDirLister::errorCode() const
+QString KDirListerTest::tempPath() const
 {
-    return m_errorCode;
-}
-
-void MyDirLister::handleError(KIO::Job *job)
-{
-    if (!m_errorExpected) {
-        qCritical() << "KDirLister called handleError!" << job << job->error() << job->errorString();
-        qFatal("aborting");
-    } else {
-        m_errorCode = job->error();
-        m_errorExpected = false;
-    }
+    return m_tempDir->path() + '/';
 }
 
 void KDirListerTest::initTestCase()
@@ -50,7 +45,11 @@ void KDirListerTest::initTestCase()
     // To avoid failing on broken locally defined MIME types
     QStandardPaths::setTestModeEnabled(true);
 
-    KIO::setDefaultJobUiDelegateExtension(nullptr); // no "skip" dialogs
+    m_tempDir.reset(new QTemporaryDir(homeTmpDir()));
+
+    // No message dialogs
+    KIO::setDefaultJobUiDelegateExtension(nullptr);
+    KIO::setDefaultJobUiDelegateFactory(nullptr);
 
     m_exitCount = 1;
 
@@ -66,12 +65,11 @@ void KDirListerTest::initTestCase()
      * PATH/subdir/subsubdir
      * PATH/subdir/subsubdir/testfile
      */
-    const QString path = m_tempDir.path() + '/';
-    createTestFile(path + "toplevelfile_1");
-    createTestFile(path + "toplevelfile_2");
-    createTestFile(path + "toplevelfile_3");
-    createTestDirectory(path + "subdir");
-    createTestDirectory(path + "subdir/subsubdir");
+    createTestFile(tempPath() + "toplevelfile_1");
+    createTestFile(tempPath() + "toplevelfile_2");
+    createTestFile(tempPath() + "toplevelfile_3");
+    createTestDirectory(tempPath() + "subdir");
+    createTestDirectory(tempPath() + "subdir/subsubdir");
 
     qRegisterMetaType<QList<QPair<KFileItem, KFileItem>>>();
 }
@@ -84,20 +82,20 @@ void KDirListerTest::cleanup()
 
 void KDirListerTest::testInvalidUrl()
 {
-    m_dirLister.setErrorExpected();
     m_dirLister.openUrl(QUrl(":/"), KDirLister::NoFlags);
     QCOMPARE(m_dirLister.spyStarted.count(), 1);
-    QTRY_COMPARE(m_dirLister.errorCode(), KIO::ERR_MALFORMED_URL);
+    QVERIFY(m_dirLister.spyJobError.wait());
+    QCOMPARE(m_dirLister.spyJobError.at(0).at(0).value<KIO::Job *>()->error(), KIO::ERR_MALFORMED_URL);
     QCOMPARE(m_dirLister.spyCompleted.count(), 0);
     QVERIFY(m_dirLister.isFinished());
 }
 
 void KDirListerTest::testNonListableUrl()
 {
-    m_dirLister.setErrorExpected();
     m_dirLister.openUrl(QUrl("data:foo"), KDirLister::NoFlags);
     QCOMPARE(m_dirLister.spyStarted.count(), 1);
-    QTRY_COMPARE(m_dirLister.errorCode(), KIO::ERR_UNSUPPORTED_ACTION);
+    QVERIFY(m_dirLister.spyJobError.wait());
+    QCOMPARE(m_dirLister.spyJobError.at(0).at(0).value<KIO::Job *>()->error(), KIO::ERR_UNSUPPORTED_ACTION);
     QCOMPARE(m_dirLister.spyCompleted.count(), 0);
     QVERIFY(m_dirLister.isFinished());
 }
@@ -105,7 +103,7 @@ void KDirListerTest::testNonListableUrl()
 void KDirListerTest::testOpenUrl()
 {
     m_items.clear();
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
     connect(&m_dirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
     // The call to openUrl itself, emits started
     m_dirLister.openUrl(QUrl::fromLocalFile(path), KDirLister::NoFlags);
@@ -178,7 +176,7 @@ void KDirListerTest::testOpenUrlFromCache()
     // Get into the case where another dirlister is holding the items
     {
         m_items.clear();
-        const QString path = m_tempDir.path() + '/';
+        const QString path = tempPath();
         MyDirLister secondDirLister;
         connect(&secondDirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
 
@@ -224,7 +222,7 @@ void KDirListerTest::testOpenUrlFromCache()
 void KDirListerTest::testNewItem()
 {
     QCOMPARE(m_items.count(), 4);
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
     connect(&m_dirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
 
     qDebug() << "Creating a new file";
@@ -260,7 +258,7 @@ void KDirListerTest::testNewItems()
     QCOMPARE(m_items.count(), 5);
     connect(&m_dirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
 
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
 
     qDebug() << "Creating 100 new files";
     for (int i = 50; i > 0; i--) {
@@ -298,7 +296,7 @@ void KDirListerTest::testNewItems()
 void KDirListerTest::benchFindByUrl()
 {
     // The time used should be in the order of O(100*log2(100))
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
     QBENCHMARK {
         for (int i = 100; i > 0; i--) {
             KFileItem cachedItem = m_dirLister.findByUrl(QUrl::fromLocalFile(path + QString("toplevelfile_new_%1").arg(i)));
@@ -313,7 +311,7 @@ void KDirListerTest::testNewItemByCopy()
     // Useful for testing #192185, i.e. whether we catch the kdirwatch event and avoid
     // a KFileItem::refresh().
     const int origItemCount = m_items.count();
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
     connect(&m_dirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
 
     QTest::qWait(1000); // We need a 1s timestamp difference on the dir, otherwise FAM won't notice anything.
@@ -351,7 +349,7 @@ void KDirListerTest::testNewItemsInSymlink() // #213799
 {
     const int origItemCount = m_items.count();
     QCOMPARE(fileCount(), origItemCount);
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
     QTemporaryFile tempFile;
     QVERIFY(tempFile.open());
     const QString symPath = tempFile.fileName() + "_link";
@@ -378,9 +376,10 @@ void KDirListerTest::testNewItemsInSymlink() // #213799
     const QString fileName = QStringLiteral("toplevelfile_newinlink");
     createSimpleFile(path + fileName);
 
-#if WORKAROUND_BROKEN_INOTIFY
-    org::kde::KDirNotify::emitFilesAdded(path);
-#endif
+    if (s_workaroundBrokenInotify) {
+        org::kde::KDirNotify::emitFilesAdded(QUrl::fromLocalFile(path));
+    }
+
     // Give time for KDirWatch to notify us
     QTRY_COMPARE(m_items2.count(), origItemCount + 1);
     QTRY_COMPARE(m_items.count(), origItemCount + 1);
@@ -418,7 +417,7 @@ void KDirListerTest::testRefreshItems()
 {
     m_refreshedItems.clear();
 
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
     const QString fileName = path + "toplevelfile_1";
     KFileItem cachedItem = m_dirLister.findByUrl(QUrl::fromLocalFile(fileName));
     QVERIFY(!cachedItem.isNull());
@@ -471,7 +470,7 @@ void KDirListerTest::testRefreshRootItem()
     // The item will be the root item of dirLister2, but also a child item
     // of m_dirLister.
     // In #190535 it would show "." instead of the subdir name, after a refresh...
-    const QString path = m_tempDir.path() + '/' + "subdir";
+    const QString path = tempPath() + "subdir";
     MyDirLister dirLister2;
     fillDirLister2(dirLister2, path);
 
@@ -540,11 +539,11 @@ void KDirListerTest::testRefreshRootItem()
 
 void KDirListerTest::testDeleteItem()
 {
-    testOpenUrl(); // ensure m_items is uptodate
+    testOpenUrl(); // ensure m_items is up-to-date
 
     const int origItemCount = m_items.count();
     QCOMPARE(fileCount(), origItemCount);
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
 
     // qDebug() << "Removing " << path+"toplevelfile_new";
     QFile::remove(path + QString("toplevelfile_new"));
@@ -569,11 +568,11 @@ void KDirListerTest::testDeleteItem()
 
 void KDirListerTest::testDeleteItems()
 {
-    testOpenUrl(); // ensure m_items is uptodate
+    testOpenUrl(); // ensure m_items is up-to-date
 
     const int origItemCount = m_items.count();
     QCOMPARE(fileCount(), origItemCount);
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
 
     qDebug() << "Removing 100 files from " << path;
     for (int i = 0; i <= 100; ++i) {
@@ -599,7 +598,7 @@ void KDirListerTest::testDeleteItems()
 void KDirListerTest::testRenameItem()
 {
     m_refreshedItems2.clear();
-    const QString dirPath = m_tempDir.path() + '/';
+    const QString dirPath = tempPath();
     connect(&m_dirLister, &KCoreDirLister::refreshItems, this, &KDirListerTest::slotRefreshItems2);
     const QString path = dirPath + "toplevelfile_2";
     const QString newPath = dirPath + "toplevelfile_2.renamed.cpp";
@@ -629,12 +628,14 @@ void KDirListerTest::testRenameItem()
 void KDirListerTest::testRenameAndOverwrite() // has to be run after testRenameItem
 {
     // Rename toplevelfile_2.renamed.html to toplevelfile_2, overwriting it.
-    const QString dirPath = m_tempDir.path() + '/';
+    const QString dirPath = tempPath();
     const QString path = dirPath + "toplevelfile_2";
     createTestFile(path);
-#if WORKAROUND_BROKEN_INOTIFY
-    org::kde::KDirNotify::emitFilesAdded(dirPath);
-#endif
+
+    if (s_workaroundBrokenInotify) {
+        org::kde::KDirNotify::emitFilesAdded(QUrl::fromLocalFile(dirPath));
+    }
+
     KFileItem existingItem;
     while (existingItem.isNull()) {
         QTest::qWait(100);
@@ -681,7 +682,7 @@ void KDirListerTest::testConcurrentListing()
 
     MyDirLister dirLister2;
 
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
 
     connect(&m_dirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
     connect(&dirLister2, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems2);
@@ -776,7 +777,7 @@ void KDirListerTest::testConcurrentHoldingListing()
 
     m_items.clear();
     m_items2.clear();
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
     MyDirLister dirLister2;
     connect(&dirLister2, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems2);
 
@@ -834,7 +835,7 @@ void KDirListerTest::testConcurrentListingAndStop()
     MyDirLister dirLister2;
 
     // Use a new tempdir for this test, so that we don't use the cache at all.
-    QTemporaryDir tempDir;
+    QTemporaryDir tempDir(homeTmpDir());
     const QString path = tempDir.path() + '/';
     createTestFile(path + "file_1");
     createTestFile(path + "file_2");
@@ -924,7 +925,7 @@ void KDirListerTest::testDeleteListerEarly()
     // qDebug() << "==========================================";
     {
         m_items.clear();
-        const QString path = m_tempDir.path() + '/';
+        const QString path = tempPath();
         MyDirLister secondDirLister;
         secondDirLister.openUrl(QUrl::fromLocalFile(path), KDirLister::NoFlags);
         QVERIFY(!secondDirLister.isFinished());
@@ -941,7 +942,7 @@ void KDirListerTest::testOpenUrlTwice()
     // Calling openUrl(reload)+openUrl(normal) before listing even starts.
     const int origItemCount = m_items.count();
     m_items.clear();
-    const QString path = m_tempDir.path() + '/';
+    const QString path = tempPath();
     MyDirLister secondDirLister;
     connect(&secondDirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
 
@@ -981,7 +982,7 @@ void KDirListerTest::testOpenUrlTwiceWithKeep()
     // (slotLoadingFinished -> setCurrentItem -> expandToUrl -> listDir),
     // which messed things up in kdirlister (unexpected reentrancy).
     m_items.clear();
-    const QString path = m_tempDir.path() + "/newsubdir";
+    const QString path = tempPath() + "newsubdir";
     QDir().mkdir(path);
     MyDirLister secondDirLister;
     connect(&secondDirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
@@ -1047,14 +1048,14 @@ void KDirListerTest::testBug211472()
 {
     m_items.clear();
 
-    QTemporaryDir newDir;
+    QTemporaryDir newDir(homeTmpDir());
     const QString path = newDir.path() + "/newsubdir/";
     QDir().mkdir(path);
     MyDirLister dirLister;
     connect(&dirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
 
     dirLister.openUrl(QUrl::fromLocalFile(path));
-    QSignalSpy spyCompleted(&dirLister, QOverload<>::of(&KCoreDirLister::completed));
+    QSignalSpy spyCompleted(&dirLister, qOverload<>(&KCoreDirLister::completed));
     QVERIFY(spyCompleted.wait(1000));
     QVERIFY(dirLister.isFinished());
     QVERIFY(m_items.isEmpty());
@@ -1097,7 +1098,7 @@ void KDirListerTest::testBug211472()
     QTRY_COMPARE(m_items.count(), 1);
 
     newDir.remove();
-    QSignalSpy spyClear(&dirLister, QOverload<>::of(&KCoreDirLister::clear));
+    QSignalSpy spyClear(&dirLister, qOverload<>(&KCoreDirLister::clear));
     QVERIFY(spyClear.wait(1000));
 }
 
@@ -1105,22 +1106,22 @@ void KDirListerTest::testRenameCurrentDir() // #294445
 {
     m_items.clear();
 
-    const QString path = m_tempDir.path() + "/newsubdir-1";
+    const QString path = tempPath() + "newsubdir-1";
     QVERIFY(QDir().mkdir(path));
     MyDirLister secondDirLister;
     connect(&secondDirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
 
     secondDirLister.openUrl(QUrl::fromLocalFile(path));
-    QSignalSpy spyCompleted(&secondDirLister, QOverload<>::of(&KCoreDirLister::completed));
+    QSignalSpy spyCompleted(&secondDirLister, qOverload<>(&KCoreDirLister::completed));
     QVERIFY(spyCompleted.wait(1000));
     QVERIFY(secondDirLister.isFinished());
     QVERIFY(m_items.empty());
     QCOMPARE(secondDirLister.rootItem().url().toLocalFile(), path);
 
-    const QString newPath = m_tempDir.path() + "/newsubdir-2";
+    const QString newPath = tempPath() + "newsubdir-2";
     QVERIFY(QDir().rename(path, newPath));
     org::kde::KDirNotify::emitFileRenamed(QUrl::fromLocalFile(path), QUrl::fromLocalFile(newPath));
-    QSignalSpy spyRedirection(&secondDirLister, QOverload<const QUrl &, const QUrl &>::of(&KCoreDirLister::redirection));
+    QSignalSpy spyRedirection(&secondDirLister, qOverload<const QUrl &, const QUrl &>(&KCoreDirLister::redirection));
     QVERIFY(spyRedirection.wait(1000));
 
     // Check that the URL of the root item got updated
@@ -1141,17 +1142,17 @@ void KDirListerTest::slotOpenUrlOnRename(const QUrl &newUrl)
 void KDirListerTest::testRenameCurrentDirOpenUrl()
 {
     m_items.clear();
-    const QString path = m_tempDir.path() + "/newsubdir-1/";
+    const QString path = tempPath() + "newsubdir-1/";
     QVERIFY(QDir().mkdir(path));
     connect(&m_dirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
 
     m_dirLister.openUrl(QUrl::fromLocalFile(path));
-    QSignalSpy spyCompleted(&m_dirLister, QOverload<>::of(&KCoreDirLister::completed));
+    QSignalSpy spyCompleted(&m_dirLister, qOverload<>(&KCoreDirLister::completed));
     // Wait for the signal completed to be emitted
     QVERIFY(spyCompleted.wait(1000));
     QVERIFY(m_dirLister.isFinished());
 
-    const QString newPath = m_tempDir.path() + "/newsubdir-2";
+    const QString newPath = tempPath() + "newsubdir-2";
     QVERIFY(QDir().rename(path, newPath));
 
     org::kde::KDirNotify::emitFileRenamed(QUrl::fromLocalFile(path), QUrl::fromLocalFile(newPath));
@@ -1159,9 +1160,9 @@ void KDirListerTest::testRenameCurrentDirOpenUrl()
     // Connect the redirection to openURL, so that on a rename the new location is opened.
     // This matches usage in gwenview, and crashes
 #if KIOCORE_BUILD_DEPRECATED_SINCE(5, 80)
-    connect(&m_dirLister, QOverload<const QUrl &>::of(&KCoreDirLister::redirection), this, &KDirListerTest::slotOpenUrlOnRename);
+    connect(&m_dirLister, qOverload<const QUrl &>(&KCoreDirLister::redirection), this, &KDirListerTest::slotOpenUrlOnRename);
 #else
-    connect(&m_dirLister, QOverload<const QUrl &, const QUrl &>::of(&KCoreDirLister::redirection), this, [this](const QUrl &, const QUrl &newUrl) {
+    connect(&m_dirLister, qOverload<const QUrl &, const QUrl &>(&KCoreDirLister::redirection), this, [this](const QUrl &, const QUrl &newUrl) {
         slotOpenUrlOnRename(newUrl);
     });
 #endif
@@ -1227,13 +1228,13 @@ void KDirListerTest::testListEmptyDirFromCache() // #278431
 {
     m_items.clear();
 
-    QTemporaryDir newDir;
+    QTemporaryDir newDir(homeTmpDir());
     const QUrl url = QUrl::fromLocalFile(newDir.path());
 
     // List and watch an empty dir
     connect(&m_dirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
     m_dirLister.openUrl(url);
-    QSignalSpy spyCompleted(&m_dirLister, QOverload<>::of(&KCoreDirLister::completed));
+    QSignalSpy spyCompleted(&m_dirLister, qOverload<>(&KCoreDirLister::completed));
     QVERIFY(spyCompleted.wait(1000));
     QVERIFY(m_dirLister.isFinished());
     QVERIFY(m_items.isEmpty());
@@ -1251,10 +1252,10 @@ void KDirListerTest::testListEmptyDirFromCache() // #278431
     // due to the if (!itemU->lstItems.isEmpty()) check which is now removed.
 
     QVERIFY(!secondDirLister.isFinished()); // we didn't go to the event loop yet
-    QSignalSpy spySecondCompleted(&secondDirLister, QOverload<>::of(&KCoreDirLister::completed));
+    QSignalSpy spySecondCompleted(&secondDirLister, qOverload<>(&KCoreDirLister::completed));
     QVERIFY(spySecondCompleted.wait(1000));
     if (!thirdDirLister.isFinished()) {
-        QSignalSpy spyThirdCompleted(&thirdDirLister, QOverload<>::of(&KCoreDirLister::completed));
+        QSignalSpy spyThirdCompleted(&thirdDirLister, qOverload<>(&KCoreDirLister::completed));
         QVERIFY(spyThirdCompleted.wait(1000));
     }
 }
@@ -1263,13 +1264,13 @@ void KDirListerTest::testWatchingAfterCopyJob() // #331582
 {
     m_items.clear();
 
-    QTemporaryDir newDir;
+    QTemporaryDir newDir(homeTmpDir());
     const QString path = newDir.path() + '/';
 
     // List and watch an empty dir
     connect(&m_dirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
     m_dirLister.openUrl(QUrl::fromLocalFile(path));
-    QSignalSpy spyCompleted(&m_dirLister, QOverload<>::of(&KCoreDirLister::completed));
+    QSignalSpy spyCompleted(&m_dirLister, qOverload<>(&KCoreDirLister::completed));
     QVERIFY(spyCompleted.wait(1000));
     QVERIFY(m_dirLister.isFinished());
     QVERIFY(m_items.isEmpty());
@@ -1307,7 +1308,7 @@ void KDirListerTest::testWatchingAfterCopyJob() // #331582
     QTRY_COMPARE(m_items.count(), 1);
 
     newDir.remove();
-    QSignalSpy clearSpy(&m_dirLister, QOverload<>::of(&KCoreDirLister::clear));
+    QSignalSpy clearSpy(&m_dirLister, qOverload<>(&KCoreDirLister::clear));
     QVERIFY(clearSpy.wait(1000));
 }
 
@@ -1315,13 +1316,13 @@ void KDirListerTest::testRemoveWatchedDirectory()
 {
     m_items.clear();
 
-    QTemporaryDir newDir;
+    QTemporaryDir newDir(homeTmpDir());
     const QString path = newDir.path() + '/';
 
     // List and watch an empty dir
     connect(&m_dirLister, &KCoreDirLister::newItems, this, &KDirListerTest::slotNewItems);
     m_dirLister.openUrl(QUrl::fromLocalFile(path));
-    QSignalSpy spyCompleted(&m_dirLister, QOverload<>::of(&KCoreDirLister::completed));
+    QSignalSpy spyCompleted(&m_dirLister, qOverload<>(&KCoreDirLister::completed));
     QVERIFY(spyCompleted.wait(1000));
     QTRY_VERIFY(m_dirLister.isFinished());
     QTRY_VERIFY(m_items.isEmpty());
@@ -1356,7 +1357,7 @@ void KDirListerTest::testRemoveWatchedDirectory()
 
 void KDirListerTest::testDirPermissionChange()
 {
-    QTemporaryDir tempDir;
+    QTemporaryDir tempDir(homeTmpDir());
 
     const QString path = tempDir.path() + '/';
     const QString subdir = path + QLatin1String("subdir");
@@ -1364,7 +1365,7 @@ void KDirListerTest::testDirPermissionChange()
 
     MyDirLister mylister;
     mylister.openUrl(QUrl::fromLocalFile(tempDir.path()));
-    QSignalSpy spyCompleted(&mylister, QOverload<>::of(&KCoreDirLister::completed));
+    QSignalSpy spyCompleted(&mylister, qOverload<>(&KCoreDirLister::completed));
     QVERIFY(spyCompleted.wait(1000));
 
     KFileItemList list = mylister.items();
@@ -1407,41 +1408,41 @@ void KDirListerTest::slotRefreshItems2(const QList<QPair<KFileItem, KFileItem>>
 
 void KDirListerTest::testCopyAfterListingAndMove() // #353195
 {
-    const QString dirA = m_tempDir.path() + "/a";
+    const QString dirA = tempPath() + "a";
     QVERIFY(QDir().mkdir(dirA));
-    const QString dirB = m_tempDir.path() + "/b";
+    const QString dirB = tempPath() + "b";
     QVERIFY(QDir().mkdir(dirB));
 
     // ensure m_dirLister holds the items.
-    m_dirLister.openUrl(QUrl::fromLocalFile(path()), KDirLister::NoFlags);
-    QSignalSpy spyCompleted(&m_dirLister, QOverload<>::of(&KCoreDirLister::completed));
+    m_dirLister.openUrl(QUrl::fromLocalFile(tempPath()), KDirLister::NoFlags);
+    QSignalSpy spyCompleted(&m_dirLister, qOverload<>(&KCoreDirLister::completed));
     QVERIFY(spyCompleted.wait());
 
     // Move b into a
     KIO::Job *moveJob = KIO::move(QUrl::fromLocalFile(dirB), QUrl::fromLocalFile(dirA));
     moveJob->setUiDelegate(nullptr);
     QVERIFY(moveJob->exec());
-    QVERIFY(QFileInfo(m_tempDir.path() + "/a/b").isDir());
+    QVERIFY(QFileInfo(tempPath() + "a/b").isDir());
 
     // Give some time to processPendingUpdates
     QTest::qWait(1000);
 
     // Copy folder a elsewhere
-    const QString dest = m_tempDir.path() + "/subdir";
+    const QString dest = tempPath() + "subdir";
     KIO::Job *copyJob = KIO::copy(QUrl::fromLocalFile(dirA), QUrl::fromLocalFile(dest));
     copyJob->setUiDelegate(nullptr);
     QVERIFY(copyJob->exec());
-    QVERIFY(QFileInfo(m_tempDir.path() + "/subdir/a/b").isDir());
+    QVERIFY(QFileInfo(tempPath() + "subdir/a/b").isDir());
 }
 
 void KDirListerTest::testRenameDirectory() // #401552
 {
     // Create the directory structure to reproduce the bug in a reliable way
-    const QString dirW = m_tempDir.path() + "/w";
+    const QString dirW = tempPath() + "w";
     QVERIFY(QDir().mkdir(dirW));
-    const QString dirW1 = m_tempDir.path() + "/w/Files";
+    const QString dirW1 = tempPath() + "w/Files";
     QVERIFY(QDir().mkdir(dirW1));
-    const QString dirW2 = m_tempDir.path() + "/w/Files/Files";
+    const QString dirW2 = tempPath() + "w/Files/Files";
     QVERIFY(QDir().mkdir(dirW2));
     // Place some empty files in each directory
     for (int i = 0; i < 50; i++) {
@@ -1474,9 +1475,9 @@ void KDirListerTest::testRenameDirectory() // #401552
     KIO::SimpleJob *job = nullptr;
     // Connect the redirection to openURL, so that on a rename the new location is opened.
 #if KIOCORE_BUILD_DEPRECATED_SINCE(5, 80)
-    connect(&m_dirLister, QOverload<const QUrl &>::of(&KCoreDirLister::redirection), this, &KDirListerTest::slotOpenUrlOnRename);
+    connect(&m_dirLister, qOverload<const QUrl &>(&KCoreDirLister::redirection), this, &KDirListerTest::slotOpenUrlOnRename);
 #else
-    connect(&m_dirLister, QOverload<const QUrl &, const QUrl &>::of(&KCoreDirLister::redirection), this, [this](const QUrl &, const QUrl &newUrl) {
+    connect(&m_dirLister, qOverload<const QUrl &, const QUrl &>(&KCoreDirLister::redirection), this, [this](const QUrl &, const QUrl &newUrl) {
         slotOpenUrlOnRename(newUrl);
     });
 #endif
@@ -1494,12 +1495,61 @@ void KDirListerTest::testRenameDirectory() // #401552
     disconnect(&m_dirLister, nullptr, this, nullptr);
 }
 
+void KDirListerTest::testRequestMimeType()
+{
+    // Use a new tempdir and lister instance for this test, so that we don't use any cache at all.
+    QTemporaryDir tempDir(homeTmpDir());
+    QString path = tempDir.path() + QLatin1Char('/');
+
+    createTestFile(path + "/file_1");
+    createTestFile(path + "/file_2.txt");
+    createTestFile(path + "/file_3.cpp");
+    createTestFile(path + "/file_3.md");
+
+    MyDirLister lister;
+    // Explicitly set requestMimeTypeWhileListing to false so we know what state
+    // it is in.
+    lister.setRequestMimeTypeWhileListing(false);
+    lister.openUrl(QUrl::fromLocalFile(path), KDirLister::NoFlags);
+
+    QTRY_VERIFY(lister.isFinished());
+
+    auto items = lister.items();
+    for (auto item : std::as_const(items)) {
+        QVERIFY(!item.isMimeTypeKnown());
+    }
+
+    // Verify that the mime types are what we expect them to be
+    QCOMPARE(items[0].mimetype(), QStringLiteral("application/octet-stream"));
+    QCOMPARE(items[1].mimetype(), QStringLiteral("text/plain"));
+    QCOMPARE(items[2].mimetype(), QStringLiteral("text/x-c++src"));
+    QCOMPARE(items[3].mimetype(), QStringLiteral("text/markdown"));
+
+    lister.setRequestMimeTypeWhileListing(true);
+    lister.openUrl(QUrl::fromLocalFile(path), KDirLister::Reload);
+
+    QTRY_VERIFY(lister.isFinished());
+
+    // If requestMimeTypeWhileListing is on, we should know the mime type of
+    // items when they have been listed.
+    items = lister.items();
+    for (auto item : std::as_const(items)) {
+        QVERIFY(item.isMimeTypeKnown());
+    }
+
+    // Verify that the mime types are what we expect them to be
+    QCOMPARE(items[0].mimetype(), QStringLiteral("application/octet-stream"));
+    QCOMPARE(items[1].mimetype(), QStringLiteral("text/plain"));
+    QCOMPARE(items[2].mimetype(), QStringLiteral("text/x-c++src"));
+    QCOMPARE(items[3].mimetype(), QStringLiteral("text/markdown"));
+}
+
 void KDirListerTest::testDeleteCurrentDir()
 {
     // ensure m_dirLister holds the items.
-    m_dirLister.openUrl(QUrl::fromLocalFile(path()), KDirLister::NoFlags);
+    m_dirLister.openUrl(QUrl::fromLocalFile(tempPath()), KDirLister::NoFlags);
     m_dirLister.clearSpies();
-    KIO::DeleteJob *job = KIO::del(QUrl::fromLocalFile(path()), KIO::HideProgressInfo);
+    KIO::DeleteJob *job = KIO::del(QUrl::fromLocalFile(tempPath()), KIO::HideProgressInfo);
     bool ok = job->exec();
     QVERIFY(ok);
     QTRY_COMPARE(m_dirLister.spyClear.count(), 1);
@@ -1514,14 +1564,14 @@ void KDirListerTest::testDeleteCurrentDir()
         deletedUrls += m_dirLister.spyItemsDeleted[i][0].value<KFileItemList>().urlList();
     }
     // qDebug() << deletedUrls;
-    QUrl currentDirUrl = QUrl::fromLocalFile(path()).adjusted(QUrl::StripTrailingSlash);
+    QUrl currentDirUrl = QUrl::fromLocalFile(tempPath()).adjusted(QUrl::StripTrailingSlash);
     // Sometimes I get ("current/subdir", "current") here, but that seems ok.
     QVERIFY(deletedUrls.contains(currentDirUrl));
 }
 
 int KDirListerTest::fileCount() const
 {
-    return QDir(path()).entryList(QDir::AllEntries | QDir::NoDotAndDotDot).count();
+    return QDir(tempPath()).entryList(QDir::AllEntries | QDir::NoDotAndDotDot).count();
 }
 
 void KDirListerTest::createSimpleFile(const QString &fileName)
