@@ -14,17 +14,14 @@
 #include <QTemporaryDir>
 #include <kdirlister.h>
 
+#include <memory>
+
 Q_DECLARE_METATYPE(KFileItemList)
 
 class GlobalInits
 {
 public:
-    GlobalInits()
-    {
-        // Must be done before the QSignalSpys connect
-        qRegisterMetaType<KFileItem>();
-        qRegisterMetaType<KFileItemList>();
-    }
+    GlobalInits();
 };
 
 class MyDirLister : public KDirLister, GlobalInits
@@ -33,23 +30,24 @@ public:
     MyDirLister()
         : spyStarted(this, &KCoreDirLister::started)
         , spyItemsDeleted(this, &KCoreDirLister::itemsDeleted)
-        , spyClear(this, QOverload<>::of(&KCoreDirLister::clear))
+        , spyClear(this, qOverload<>(&KCoreDirLister::clear))
         , spyClearDir(this, &KCoreDirLister::clearDir)
-        , spyCompleted(this, QOverload<>::of(&KCoreDirLister::completed))
-        , spyCanceled(this, QOverload<>::of(&KCoreDirLister::canceled))
+        , spyCompleted(this, qOverload<>(&KCoreDirLister::completed))
+        , spyCanceled(this, qOverload<>(&KCoreDirLister::canceled))
 #if KIOCORE_BUILD_DEPRECATED_SINCE(5, 79)
-        , spyClearQUrl(this, QOverload<const QUrl &>::of(&KCoreDirLister::clear))
-        , spyCompletedQUrl(this, QOverload<const QUrl &>::of(&KCoreDirLister::completed))
-        , spyCanceledQUrl(this, QOverload<const QUrl &>::of(&KCoreDirLister::canceled))
+        , spyClearQUrl(this, qOverload<const QUrl &>(&KCoreDirLister::clear))
+        , spyCompletedQUrl(this, qOverload<const QUrl &>(&KCoreDirLister::completed))
+        , spyCanceledQUrl(this, qOverload<const QUrl &>(&KCoreDirLister::canceled))
 #else
         , spyCompletedQUrl(this, &KCoreDirLister::listingDirCompleted)
         , spyCanceledQUrl(this, &KCoreDirLister::listingDirCanceled)
 #endif
 #if KIOCORE_BUILD_DEPRECATED_SINCE(5, 80)
-        , spyRedirection(this, QOverload<const QUrl &>::of(&KCoreDirLister::redirection))
+        , spyRedirection(this, qOverload<const QUrl &>(&KCoreDirLister::redirection))
 #else
-        , spyRedirection(this, QOverload<const QUrl &, const QUrl &>::of(&KCoreDirLister::redirection))
+        , spyRedirection(this, qOverload<const QUrl &, const QUrl &>(&KCoreDirLister::redirection))
 #endif
+        , spyJobError(this, &KCoreDirLister::jobError)
     {
     }
 
@@ -66,6 +64,7 @@ public:
         spyCanceledQUrl.clear();
         spyRedirection.clear();
         spyItemsDeleted.clear();
+        spyJobError.clear();
     }
 
     QSignalSpy spyStarted;
@@ -80,15 +79,7 @@ public:
     QSignalSpy spyCompletedQUrl;
     QSignalSpy spyCanceledQUrl;
     QSignalSpy spyRedirection;
-
-    void setErrorExpected();
-    int errorCode() const;
-
-protected:
-    void handleError(KIO::Job *job) override;
-
-    bool m_errorExpected = false;
-    int m_errorCode = 0;
+    QSignalSpy spyJobError;
 };
 
 class KDirListerTest : public QObject
@@ -129,6 +120,7 @@ private Q_SLOTS:
     void testDirPermissionChange();
     void testCopyAfterListingAndMove(); // #353195
     void testRenameDirectory(); // #401552
+    void testRequestMimeType();
     void testDeleteCurrentDir(); // must be last!
 
 protected Q_SLOTS: // 'more private than private slots' - i.e. not seen by qtestlib
@@ -143,10 +135,7 @@ Q_SIGNALS:
 
 private:
     int fileCount() const;
-    QString path() const
-    {
-        return m_tempDir.path() + '/';
-    }
+    QString tempPath() const;
     void createSimpleFile(const QString &fileName);
     void fillDirLister2(MyDirLister &lister, const QString &path);
     void waitUntilMTimeChange(const QString &path);
@@ -155,7 +144,7 @@ private:
 private:
     int m_exitCount;
     QEventLoop m_eventLoop;
-    QTemporaryDir m_tempDir;
+    std::unique_ptr<QTemporaryDir> m_tempDir;
     MyDirLister m_dirLister;
     KFileItemList m_items;
     KFileItemList m_items2;
