@@ -12,11 +12,12 @@
 #include "kiogui_export.h"
 #include <kio/job_base.h> // for LoadType
 
+#include <memory>
+
 class QUrl;
 
 namespace KIO
 {
-
 class FavIconRequestJobPrivate;
 /**
  * @class FavIconRequestJob faviconrequestjob.h <KIO/FavIconRequestJob>
@@ -72,7 +73,7 @@ public:
      * Destructor. You do not need to delete the job, it will delete automatically,
      * unless you call setAutoDelete(false).
      */
-    ~FavIconRequestJob();
+    ~FavIconRequestJob() override;
 
     /**
      * @brief setIconUrl allows to set, for a specific URL, a different icon URL
@@ -103,7 +104,9 @@ public:
      * @internal
      * Do not call start(), KIO jobs are autostarted
      */
-    void start() override {}
+    void start() override
+    {
+    }
 
 private Q_SLOTS:
     void doStart(); // not called start() so that exec() doesn't call it too
@@ -112,7 +115,7 @@ private Q_SLOTS:
 private:
     Q_PRIVATE_SLOT(d, void slotData(KIO::Job *, const QByteArray &))
 
-    FavIconRequestJobPrivate *const d;
+    std::unique_ptr<FavIconRequestJobPrivate> const d;
 };
 
 }
