@@ -10,7 +10,7 @@
 #include "kfileitem.h"
 
 #include "joburlcache_p.h"
-#include <KIconLoader>
+#include <KIconUtils>
 #include <KJobUiDelegate>
 #include <KLocalizedString>
 #include <KUrlMimeData>
@@ -276,7 +276,7 @@ void KDirModelPrivate::removeFromNodeHash(KDirModelNode *node, const QUrl &url)
     if (node->item().isDir()) {
         QList<QUrl> urls;
         static_cast<KDirModelDirNode *>(node)->collectAllChildUrls(urls);
-        for (const QUrl &u : qAsConst(urls)) {
+        for (const QUrl &u : std::as_const(urls)) {
             m_nodeHash.remove(u);
         }
     }
@@ -411,10 +411,7 @@ KDirModel::KDirModel(QObject *parent)
     setDirLister(new KDirLister(this));
 }
 
-KDirModel::~KDirModel()
-{
-    delete d;
-}
+KDirModel::~KDirModel() = default;
 
 void KDirModel::setDirLister(KDirLister *dirLister)
 {
@@ -436,10 +433,10 @@ void KDirModel::setDirLister(KDirLister *dirLister)
     connect(d->m_dirLister, &KCoreDirLister::refreshItems, this, [this](const QList<QPair<KFileItem, KFileItem>> &items) {
         d->_k_slotRefreshItems(items);
     });
-    connect(d->m_dirLister, QOverload<>::of(&KCoreDirLister::clear), this, [this]() {
+    connect(d->m_dirLister, qOverload<>(&KCoreDirLister::clear), this, [this]() {
         d->_k_slotClear();
     });
-    connect(d->m_dirLister, QOverload<const QUrl &, const QUrl &>::of(&KCoreDirLister::redirection), this, [this](const QUrl &oldUrl, const QUrl &newUrl) {
+    connect(d->m_dirLister, qOverload<const QUrl &, const QUrl &>(&KCoreDirLister::redirection), this, [this](const QUrl &oldUrl, const QUrl &newUrl) {
         d->_k_slotRedirection(oldUrl, newUrl);
     });
 }
@@ -521,28 +518,26 @@ void KDirModelPrivate::_k_slotNewItems(const QUrl &directoryUrl, const KFileItem
     QList<QModelIndex> emitExpandFor;
 
     dirNode->m_childNodes.reserve(newRowCount);
-    KFileItemList::const_iterator it = items.begin();
-    KFileItemList::const_iterator end = items.end();
-    for (; it != end; ++it) {
-        const bool isDir = it->isDir();
-        KDirModelNode *node = isDir ? new KDirModelDirNode(dirNode, *it) : new KDirModelNode(dirNode, *it);
+    for (const auto &item : items) {
+        const bool isDir = item.isDir();
+        KDirModelNode *node = isDir ? new KDirModelDirNode(dirNode, item) : new KDirModelNode(dirNode, item);
 #ifndef NDEBUG
         // Test code for possible duplication of items in the childnodes list,
         // not sure if/how it ever happened.
         // if (dirNode->m_childNodes.count() &&
-        //    dirNode->m_childNodes.last()->item().name() == (*it).name()) {
-        //    qCWarning(category) << "Already having" << (*it).name() << "in" << directoryUrl
+        //    dirNode->m_childNodes.last()->item().name() == item.name()) {
+        //    qCWarning(category) << "Already having" << item.name() << "in" << directoryUrl
         //             << "url=" << dirNode->m_childNodes.last()->item().url();
         //    abort();
         //}
 #endif
         dirNode->m_childNodes.append(node);
-        const QUrl url = it->url();
+        const QUrl url = item.url();
         m_nodeHash.insert(cleanupUrl(url), node);
 
         if (!urlsBeingFetched.isEmpty()) {
             const QUrl &dirUrl = url;
-            for (const QUrl &urlFetched : qAsConst(urlsBeingFetched)) {
+            for (const QUrl &urlFetched : std::as_const(urlsBeingFetched)) {
                 if (dirUrl.matches(urlFetched, QUrl::StripTrailingSlash) || dirUrl.isParentOf(urlFetched)) {
                     // qDebug() << "Listing found" << dirUrl.url() << "which is a parent of fetched url" << urlFetched;
                     const QModelIndex parentIndex = indexForNode(node, dirNode->m_childNodes.count() - 1);
@@ -561,7 +556,7 @@ void KDirModelPrivate::_k_slotNewItems(const QUrl &directoryUrl, const KFileItem
 
     // Emit expand signal after rowsInserted signal has been emitted,
     // so that any proxy model will have updated its mapping already
-    for (const QModelIndex &idx : qAsConst(emitExpandFor)) {
+    for (const QModelIndex &idx : std::as_const(emitExpandFor)) {
         Q_EMIT q->expand(idx);
     }
 }
@@ -655,15 +650,16 @@ void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList &items)
 
 void KDirModelPrivate::_k_slotRefreshItems(const QList<QPair<KFileItem, KFileItem>> &items)
 {
-    QModelIndex topLeft, bottomRight;
+    QModelIndex topLeft;
+    QModelIndex bottomRight;
 
     // Solution 1: we could emit dataChanged for one row (if items.size()==1) or all rows
     // Solution 2: more fine-grained, actually figure out the beginning and end rows.
-    for (QList<QPair<KFileItem, KFileItem>>::const_iterator fit = items.begin(), fend = items.end(); fit != fend; ++fit) {
-        Q_ASSERT(!fit->first.isNull());
-        Q_ASSERT(!fit->second.isNull());
-        const QUrl oldUrl = fit->first.url();
-        const QUrl newUrl = fit->second.url();
+    for (const auto &[oldItem, newItem] : items) {
+        Q_ASSERT(!oldItem.isNull());
+        Q_ASSERT(!newItem.isNull());
+        const QUrl oldUrl = oldItem.url();
+        const QUrl newUrl = newItem.url();
         KDirModelNode *node = nodeForUrl(oldUrl); // O(n); maybe we could look up to the parent only once
         // qDebug() << "in model for" << m_dirLister->url() << ":" << oldUrl << "->" << newUrl << "node=" << node;
         if (!node) { // not found [can happen when renaming a dir, redirection was emitted already]
@@ -672,17 +668,17 @@ void KDirModelPrivate::_k_slotRefreshItems(const QList<QPair<KFileItem, KFileIte
         if (node != m_rootNode) { // we never set an item in the rootnode, we use m_dirLister->rootItem instead.
             bool hasNewNode = false;
             // A file became directory (well, it was overwritten)
-            if (fit->first.isDir() != fit->second.isDir()) {
+            if (oldItem.isDir() != newItem.isDir()) {
                 // qDebug() << "DIR/FILE STATUS CHANGE";
                 const int r = node->rowNumber();
                 removeFromNodeHash(node, oldUrl);
                 KDirModelDirNode *dirNode = node->parent();
                 delete dirNode->m_childNodes.takeAt(r); // i.e. "delete node"
-                node = fit->second.isDir() ? new KDirModelDirNode(dirNode, fit->second) : new KDirModelNode(dirNode, fit->second);
+                node = newItem.isDir() ? new KDirModelDirNode(dirNode, newItem) : new KDirModelNode(dirNode, newItem);
                 dirNode->m_childNodes.insert(r, node); // same position!
                 hasNewNode = true;
             } else {
-                node->setItem(fit->second);
+                node->setItem(newItem);
             }
 
             if (oldUrl != newUrl || hasNewNode) {
@@ -692,7 +688,7 @@ void KDirModelPrivate::_k_slotRefreshItems(const QList<QPair<KFileItem, KFileIte
                 m_nodeHash.insert(cleanupUrl(newUrl), node);
             }
             // MIME type changed -> forget cached icon (e.g. from "cut", #164185 comment #13)
-            if (fit->first.determineMimeType().name() != fit->second.determineMimeType().name()) {
+            if (oldItem.determineMimeType().name() != newItem.determineMimeType().name()) {
                 node->setPreview(QIcon());
             }
 
@@ -760,7 +756,7 @@ void KDirModelPrivate::_k_slotJobUrlsChanged(const QStringList &urlList)
 
     m_allCurrentDestUrls = urlList;
 
-    for (const QString &dirtyUrl : qAsConst(dirtyUrls)) {
+    for (const QString &dirtyUrl : std::as_const(dirtyUrls)) {
         if (KDirModelNode *node = nodeForUrl(QUrl(dirtyUrl))) {
             const QModelIndex idx = indexForNode(node);
             Q_EMIT q->dataChanged(idx, idx, {KDirModel::HasJobRole});
@@ -773,7 +769,7 @@ void KDirModelPrivate::clearAllPreviews(KDirModelDirNode *dirNode)
     const int numRows = dirNode->m_childNodes.count();
     if (numRows > 0) {
         KDirModelNode *lastNode = nullptr;
-        for (KDirModelNode *node : qAsConst(dirNode->m_childNodes)) {
+        for (KDirModelNode *node : std::as_const(dirNode->m_childNodes)) {
             node->setPreview(QIcon());
             // node->setPreview(QIcon::fromTheme(node->item().iconName()));
             if (isDir(node)) {
@@ -851,7 +847,20 @@ QVariant KDirModel::data(const QModelIndex &index, int role) const
                 }
                 Q_ASSERT(!item.isNull());
                 // qDebug() << item->url() << " overlays=" << item->overlays();
-                return KDE::icon(item.iconName(), item.overlays());
+                static const QIcon fallbackIcon = QIcon::fromTheme(QStringLiteral("unknown"));
+
+                const QString iconName(item.iconName());
+                QIcon icon;
+
+                if (QDir::isAbsolutePath(iconName)) {
+                    icon = QIcon(iconName);
+                }
+                if (icon.isNull()
+                    || (!(iconName.endsWith(QLatin1String(".svg")) || iconName.endsWith(QLatin1String(".svgz"))) && icon.availableSizes().isEmpty())) {
+                    icon = QIcon::fromTheme(iconName, fallbackIcon);
+                }
+
+                return KIconUtils::addOverlays(icon, item.overlays());
             }
             break;
         case Qt::TextAlignmentRole:
@@ -1092,7 +1101,8 @@ QStringList KDirModel::mimeTypes() const
 
 QMimeData *KDirModel::mimeData(const QModelIndexList &indexes) const
 {
-    QList<QUrl> urls, mostLocalUrls;
+    QList<QUrl> urls;
+    QList<QUrl> mostLocalUrls;
     urls.reserve(indexes.size());
     mostLocalUrls.reserve(indexes.size());
     bool canUseMostLocalUrls = true;
@@ -1142,8 +1152,6 @@ QModelIndex KDirModel::indexForItem(const KFileItem *item) const
 
 QModelIndex KDirModel::indexForItem(const KFileItem &item) const
 {
-    // Note that we can only use the URL here, not the pointer.
-    // KFileItems can be copied.
     return indexForUrl(item.url()); // O(n)
 }
 
