@@ -16,6 +16,7 @@
 #include <limits.h>
 #include <stdlib.h>
 
+#include <QCollator>
 #include <QDebug>
 #include <QDir>
 #include <QDirIterator>
@@ -84,6 +85,16 @@ static int initialWaitDuration()
     return s_waitDuration;
 }
 
+// For local paths we use our custom comparer function that ignores the trailing slash character
+static void sortLocalPaths(QStringList &list)
+{
+    QCollator c;
+    c.setCaseSensitivity(Qt::CaseSensitive);
+    std::sort(list.begin(), list.end(), [c](const QString &a, const QString &b) {
+        return c.compare(a.endsWith(QStringLiteral("/")) ? a.chopped(1) : a, b.endsWith(QStringLiteral("/")) ? b.chopped(1) : b) < 0;
+    });
+}
+
 ///////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////
 // KUrlCompletionPrivate
@@ -322,11 +333,14 @@ void DirectoryListThread::run()
 
     QMimeDatabase mimeTypes;
 
-    const QStringList::const_iterator end = m_dirList.constEnd();
-    for (QStringList::const_iterator it = m_dirList.constBegin(); it != end && !terminationRequested(); ++it) {
-        // qDebug() << "Scanning directory" << *it;
+    for (const QString &dir : std::as_const(m_dirList)) {
+        if (terminationRequested()) {
+            break;
+        }
 
-        QDirIterator current_dir_iterator(*it, iterator_filter);
+        // qDebug() << "Scanning directory" << dir;
+
+        QDirIterator current_dir_iterator(dir, iterator_filter);
 
         while (current_dir_iterator.hasNext() && !terminationRequested()) {
             current_dir_iterator.next();
@@ -519,7 +533,6 @@ KUrlCompletion::KUrlCompletion(Mode _mode)
 KUrlCompletion::~KUrlCompletion()
 {
     stop();
-    delete d;
 }
 
 void KUrlCompletionPrivate::init()
@@ -610,6 +623,13 @@ QString KUrlCompletion::makeCompletion(const QString &text)
 
     d->complete_url = url.isURL();
 
+    // We use our custom sorter function if we are completing local paths
+    setSorterFunction(!d->complete_url ? sortLocalPaths : nullptr);
+
+    // If we typed an exact path to a directory, we block autosuggestion
+    // or else it would autosuggest the first child dir.
+    setShouldAutoSuggest(d->complete_url || (url.dir() != text));
+
     QString aMatch;
 
     // Environment variables
@@ -1144,23 +1164,17 @@ void KUrlCompletionPrivate::_k_slotEntries(KIO::Job *, const KIO::UDSEntryList &
 {
     QStringList matchList;
 
-    KIO::UDSEntryList::ConstIterator it = entries.constBegin();
-    const KIO::UDSEntryList::ConstIterator end = entries.constEnd();
-
-    QString filter = list_urls_filter;
-
-    int filter_len = filter.length();
+    const QString filter = list_urls_filter;
+    const int filter_len = filter.length();
 
     // Iterate over all files
-    //
-    for (; it != end; ++it) {
-        const KIO::UDSEntry &entry = *it;
-        const QString url = entry.stringValue(KIO::UDSEntry::UDS_URL);
+    for (const auto &entry : entries) {
+        const QString udsUrl = entry.stringValue(KIO::UDSEntry::UDS_URL);
 
         QString entry_name;
-        if (!url.isEmpty()) {
+        if (!udsUrl.isEmpty()) {
             // qDebug() << "url:" << url;
-            entry_name = QUrl(url).fileName();
+            entry_name = QUrl(udsUrl).fileName();
         } else {
             entry_name = entry.stringValue(KIO::UDSEntry::UDS_NAME);
         }
@@ -1183,7 +1197,7 @@ void KUrlCompletionPrivate::_k_slotEntries(KIO::Job *, const KIO::UDSEntryList &
             continue;
         }
 
-        if (filter_len != 0 && entry_name.leftRef(filter_len) != filter) {
+        if (filter_len != 0 && QStringView(entry_name).left(filter_len) != filter) {
             continue;
         }
 
@@ -1328,9 +1342,7 @@ void KUrlCompletionPrivate::slotCompletionThreadDone(QThread *thread, const QStr
         thread->wait();
         delete thread;
         userListThread = nullptr;
-    }
-
-    if (dirListThread == thread) {
+    } else if (dirListThread == thread) {
         thread->wait();
         delete thread;
         dirListThread = nullptr;
@@ -1413,9 +1425,9 @@ static bool expandEnv(QString &text)
             // and defined
             //
             if (pos2 >= 0) {
-                int len = pos2 - pos;
-                const QStringRef key = text.midRef(pos + 1, len - 1);
-                QString value = QString::fromLocal8Bit(qgetenv(key.toLocal8Bit().constData()));
+                const int len = pos2 - pos;
+                const QStringView key = QStringView(text).mid(pos + 1, len - 1);
+                const QString value = QString::fromLocal8Bit(qgetenv(key.toLocal8Bit().constData()));
 
                 if (!value.isEmpty()) {
                     expanded = true;
